import { Component, trigger, style, transition, animate, keyframes } from '@angular/core';
import { Platform, NavController } from 'ionic-angular';
import { LedsRestService } from './../../services/ledsRest.service';
import { Led } from './../../models/leds';
// not used due performance issues
// import { PhytecTrafficLiveData } from './../phytec-traffic-live-data/phytec-traffic-live-data';
import { Observable } from 'rxjs/Rx';

import 'rxjs/add/observable/forkJoin';

@Component({
  selector: 'phytec-leds-rest',
  templateUrl: 'phytec-leds-rest.html',
  animations: [
    trigger('dataSent', [
      transition('void => *', [
        animate(1000, keyframes([
          style({opacity: 0, offset: 0}),
          style({opacity: 1, offset: 0.3}),
          style({opacity: 1, offset: 1.0})
        ]))
      ]),
      transition('* => void', [
        animate(1000, keyframes([
          style({opacity: 1, offset: 0}),
          style({opacity: 1, offset: 0.3}),
          style({opacity: 0, offset: 1.0})
        ]))
      ])
    ]),
  ]
})
export class PhytecLedsRest {
  // comment out due perfomance issues
  // @ViewChild('trafficLiveData') trafficLiveData: PhytecTrafficLiveData;
  private touchCount: number;
  private ledData$: Observable<Led[]>;
  private trafficIndicator = {
    showData: false,
    dataValue: 0,
    timeout: 0
  };
  private ledData: Led[] = [
    {
      id: 1,
      value: false
    },
    {
      id: 2,
      value: false
    },
    {
      id: 3,
      value: false
    }
  ];

  constructor(public platform: Platform,
              public navCtrl: NavController,
              public ledsRestService: LedsRestService) {
    this.ledData$ = ledsRestService.getLeds();
  }

  trafficHandler() {
    // simulate 415 byte data payload
    const dataPayload = 415;
    // comment out due perfomance issues
    // this.trafficLiveData.addTrafficData(dataPayload);
    this.showTrafficIndicator(dataPayload);
  }

  showTrafficIndicator(value) {
    if (this.trafficIndicator.timeout) {
      clearTimeout(this.trafficIndicator.timeout);
      this.trafficIndicator.dataValue += value;
    }
    this.trafficIndicator.showData = true;
    this.trafficIndicator.timeout = setTimeout(() => {
      this.trafficIndicator.showData = !this.trafficIndicator.showData;
      this.trafficIndicator.dataValue = 0;
    }, 2000);
  }

  updateAllLedData(dataMatrix: Array<boolean>) {
    Observable.forkJoin(
      this.ledsRestService.updateLed({id: 1, value: dataMatrix[0]}).map((item) => {
        this.trafficHandler();
        return item;
      }),
      this.ledsRestService.updateLed({id: 2, value: dataMatrix[1]}).map((item) => {
        this.trafficHandler();
        return item;
      }),
      this.ledsRestService.updateLed({id: 3, value: dataMatrix[2]}).map((item) => {
        this.trafficHandler();
        return item;
      })
    ).subscribe(result => {
      result.map(result => {
        this.updateLedLocalValues(result.id, result.value);
      });
      this.setCounter();
    });
  }

  setCounter() {
    this.touchCount = 0;
    this.ledData.map(led => {
      led.value ? this.touchCount++ : '';
    });
  }

  updateLedData(id, value, event) {
    if (value !== event.checked) {
      this.ledsRestService.updateLed({id: id, value: !value}).map((item) => {
        this.trafficHandler();
        return item;
      }).subscribe(result => {
        this.updateLedLocalValues(id, result.value);
        this.setCounter();
      });
    } else {
      this.updateLedLocalValues(id, value);
      this.setCounter();
    }
  }

  updateLedLocalValues(id, value) {
    this.ledData.map(led => {
      led.id === id ? led.value = value : '';
      return led;
    });
  }

  onTouchChange(touches) {
    switch (touches) {
      case 1:
        this.updateAllLedData([true, false, false]);
        break;
      case 2:
        this.updateAllLedData([true, true, false]);

        break;
      case 3:
        this.updateAllLedData([true, true, true]);

        break;
      case 0:
        this.updateAllLedData([false, false, false]);
      default:
        console.log('no matching touches');
    }
  }

  ionViewDidLoad() {
    this.ledData$.subscribe(data => {
      this.ledData = data;
    })
  }
}