import { Component, trigger, style, transition, animate, keyframes } from '@angular/core';
import { Platform, NavController } from 'ionic-angular';
import { LedsSocketsService } from './../../services/ledsSocket.service';
import { Led } from './../../models/leds';
// not used due performance issues
// import { PhytecTrafficLiveData } from './../phytec-traffic-live-data/phytec-traffic-live-data';

import 'rxjs/add/observable/forkJoin';

@Component({
  selector: 'phytec-leds-socket',
  templateUrl: 'phytec-leds-socket.html',
  animations: [
    trigger('dataSent', [
      transition('void => *', [
        animate(1000, keyframes([
          style({opacity: 0, offset: 0}),
          style({opacity: 1, offset: 0.3}),
          style({opacity: 1, offset: 1.0})
        ]))
      ]),
      transition('* => void', [
        animate(1000, keyframes([
          style({opacity: 1, offset: 0}),
          style({opacity: 1, offset: 0.3}),
          style({opacity: 0, offset: 1.0})
        ]))
      ])
    ]),
  ]
})
export class PhytecLedsSocket {
  // comment out due perfomance issues
  // @ViewChild('trafficLiveData') trafficLiveData: PhytecTrafficLiveData;
  private touchCount: number;
  private socket;
  private trafficIndicator = {
    showData: false,
    dataValue: 0,
    timeout: 0
  };
  private randomPushServerToggle = false;
  private ledData: Led[] = [
    {
      id: 1,
      value: false
    },
    {
      id: 2,
      value: false
    },
    {
      id: 3,
      value: false
    }
  ];

  constructor(public platform: Platform,
              public navCtrl: NavController,
              public ledsSocketsService: LedsSocketsService) {
    this.socket = ledsSocketsService.socketForDevice;
    this.socket.on('allLeds', data => {
      this.ledData = data;
      this.setCounter();
    });
    this.socket.on('led', data => {
      // simulate 48 byte data payload
      this.trafficHandler(48);
      this.updateLedLocalValues(data.id, data.value);
      this.setCounter();
    });
  }

  toggleRandomPushServer(value, event) {
    if (value !== event.checked) {
      this.randomPushServerToggle = !this.randomPushServerToggle;
    }
    this.socket.emit('random', this.randomPushServerToggle);
  }

  trafficHandler(dataPayload) {
    // comment out due perfomance issues
    // this.trafficLiveData.addTrafficData(dataPayload);
    this.showTrafficIndicator(dataPayload);
  }

  showTrafficIndicator(value) {
    if (this.trafficIndicator.timeout) {
      clearTimeout(this.trafficIndicator.timeout);
      this.trafficIndicator.dataValue += value;
    }
    this.trafficIndicator.showData = true;
    this.trafficIndicator.timeout = setTimeout(() => {
      this.trafficIndicator.showData = !this.trafficIndicator.showData;
      this.trafficIndicator.dataValue = 0;
    }, 2000);
  }

  updateAllLedData(dataMatrix: Array<boolean>) {
    this.ledsSocketsService.updateLed({id: 1, value: dataMatrix[0]});
    this.ledsSocketsService.updateLed({id: 2, value: dataMatrix[1]});
    this.ledsSocketsService.updateLed({id: 3, value: dataMatrix[2]});
  }

  setCounter() {
    this.touchCount = 0;
    this.ledData.map(led => {
      led.value ? this.touchCount++ : '';
    });
  }

  updateLedData(id, value, event) {
    if (value !== event.checked) {
      this.ledsSocketsService.updateLed({id: id, value: !value});
    }
  }

  updateLedLocalValues(id, value) {
    this.randomPushServerToggle = false;
    this.ledData.map(led => {
      led.id === id ? led.value = value : '';
      return led;
    });
  }

  onTouchChange(touches) {
    switch (touches) {
      case 1:
        this.updateAllLedData([true, false, false]);
        break;
      case 2:
        this.updateAllLedData([true, true, false]);

        break;
      case 3:
        this.updateAllLedData([true, true, true]);

        break;
      case 0:
        this.updateAllLedData([false, false, false]);
      default:
        console.log('no matching touches');
    }
  }

  ionViewDidLoad() {
  }
}