import { Component, Input, ViewChild, ElementRef, AfterViewInit } from '@angular/core';
import { Platform, NavController } from 'ionic-angular';
import { SmoothieChart, TimeSeries } from 'smoothie';

import 'rxjs/add/observable/forkJoin';

@Component({
  selector: 'phytec-traffic-live-data',
  templateUrl: 'phytec-traffic-live-data.html'
})
export class PhytecTrafficLiveData implements AfterViewInit {
  @ViewChild('canvas') private canvas: ElementRef;
  /* tslint:disable */
  @Input('height') private height: number;
  @Input('width') private width: number;
  /* tslint:enable */
  private value = 0;
  private timeSeries = new TimeSeries();

  private smoothie;

  constructor(public platform: Platform,
              public navCtrl: NavController) {
  }

  ngAfterViewInit() {
    this.smoothie = new SmoothieChart({
      maxValue: 600,
      interpolation: 'step',
      grid: {
        lineWidth: 1,
        millisPerLine: 250,
        verticalSections: 0,
        borderVisible: false,
        fillStyle: '#d3d3d3',
        strokeStyle:'#d3d3d3'
      }
    });
    this.smoothie.streamTo(this.canvas.nativeElement, 1000);
  }

  addTrafficData(value) {
    if (value !== this.value) {
      this.value = value;
      const now = new Date().getTime();
      this.timeSeries.append(now, value);
      this.smoothie.addTimeSeries(this.timeSeries, {lineWidth: 3});
      console.log(value);
      this.addTrafficData(0);
    }
  }
}